﻿<?php
/*
Plugin Name: Custom PayPal Gateway
Description: A custom PayPal Standard gateway for WooCommerce.
Version: 1.3
Author: Ralph Chastain
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

// Check if WooCommerce is active and only then proceed
add_action( 'plugins_loaded', 'init_custom_paypal_gateway' );

function init_custom_paypal_gateway() {

    if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
        return;
    }

    // Add the gateway to WooCommerce
    function add_custom_gateway_class( $methods ) {
        $methods[] = 'WC_Custom_PayPal_Gateway';
        return $methods;
    }
    add_filter( 'woocommerce_payment_gateways', 'add_custom_gateway_class' );

    // Define the custom gateway class
    class WC_Custom_PayPal_Gateway extends WC_Payment_Gateway {

        // Explicitly declare properties
        public $email;
        public $sandbox;

        public function __construct() {
            $this->id                 = 'custom_paypal';
            $this->icon               = apply_filters( 'woocommerce_custom_paypal_icon', '' );
            $this->has_fields         = false;
            $this->method_title       = __( 'Custom PayPal', 'woocommerce' );
            $this->method_description = __( 'Custom PayPal Standard payment gateway.', 'woocommerce' );

            // Load the settings
            $this->init_form_fields();
            $this->init_settings();

            $this->title        = $this->get_option( 'title' );
            $this->description  = $this->get_option( 'description' );
            $this->email        = $this->get_option( 'email' );
            $this->sandbox      = $this->get_option( 'sandbox' );

            // Add hooks
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            add_action( 'woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );
            add_action( 'woocommerce_api_wc_custom_paypal_gateway', array( $this, 'check_response' ) );
        }

        public function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title'   => __( 'Enable/Disable', 'woocommerce' ),
                    'type'    => 'checkbox',
                    'label'   => __( 'Enable Custom PayPal Gateway', 'woocommerce' ),
                    'default' => 'yes'
                ),
                'title' => array(
                    'title'       => __( 'Title', 'woocommerce' ),
                    'type'        => 'text',
                    'description' => __( 'This controls the title the user sees during checkout.', 'woocommerce' ),
                    'default'     => __( 'PayPal', 'woocommerce' ),
                    'desc_tip'    => true,
                ),
                'description' => array(
                    'title'       => __( 'Description', 'woocommerce' ),
                    'type'        => 'textarea',
                    'description' => __( 'This controls the description the user sees during checkout.', 'woocommerce' ),
                    'default'     => __( 'Pay via PayPal; you can pay with your credit card if you don’t have a PayPal account.', 'woocommerce' ),
                    'desc_tip'    => true,
                ),
                'email' => array(
                    'title'       => __( 'PayPal Email', 'woocommerce' ),
                    'type'        => 'email',
                    'description' => __( 'Please enter your PayPal email address; this is needed to take payment.', 'woocommerce' ),
                    'default'     => '',
                    'desc_tip'    => true,
                ),
                'sandbox' => array(
                    'title'       => __( 'Sandbox Mode', 'woocommerce' ),
                    'type'        => 'checkbox',
                    'label'       => __( 'Enable Sandbox Mode', 'woocommerce' ),
                    'default'     => 'yes',
                    'desc_tip'    => true,
                ),
            );
        }

        public function process_payment( $order_id ) {
            $order = wc_get_order( $order_id );

            // Mark as on-hold (we're awaiting the payment)
            $order->update_status( 'on-hold', __( 'Awaiting PayPal payment', 'woocommerce' ) );

            // Reduce stock levels
            wc_reduce_stock_levels( $order_id );

            // Remove cart
            WC()->cart->empty_cart();

            // Redirect to PayPal
            return array(
                'result'   => 'success',
                'redirect' => $this->generate_paypal_url( $order )
            );
        }

        public function receipt_page( $order ) {
            echo '<p>' . __( 'Thank you for your order, please click the button below to pay with PayPal.', 'woocommerce' ) . '</p>';
            echo '<a class="button" href="' . esc_url( $this->generate_paypal_url( $order ) ) . '">' . __( 'Pay with PayPal', 'woocommerce' ) . '</a>';
        }

        private function generate_paypal_url( $order ) {
            $paypal_url = $this->sandbox === 'yes' ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr';

            $paypal_args = array(
                'cmd'           => '_xclick',
                'business'      => $this->email,
                'amount'        => $order->get_total(),
                'currency_code' => get_woocommerce_currency(),
                'item_name'     => 'Order ' . $order->get_order_number(),
                'return'        => $this->get_return_url( $order ),
                'cancel_return' => wc_get_cart_url(),
                'notify_url'    => WC()->api_request_url( 'WC_Custom_PayPal_Gateway' ),
            );

            $paypal_args = http_build_query( $paypal_args, '', '&' );
            return $paypal_url . '?' . $paypal_args;
        }

        public function check_response() {
            @ob_clean();
            $verified = $this->validate_ipn();
            if ( $verified ) {
                header( 'HTTP/1.1 200 OK' );
                do_action( 'valid-paypal-standard-ipn-request', $_POST );
            } else {
                wp_die( "PayPal IPN Request Failure" );
            }
        }

        private function validate_ipn() {
            $paypal_url = $this->sandbox === 'yes' ? 'https://ipnpb.sandbox.paypal.com/cgi-bin/webscr' : 'https://ipnpb.paypal.com/cgi-bin/webscr';
            $post_data = wp_unslash( $_POST );
            $post_data['cmd'] = '_notify-validate';

            $params = array(
                'body' => $post_data,
                'timeout' => 60,
                'httpversion' => '1.1',
                'compress' => false,
                'decompress' => false,
                'user-agent' => 'WooCommerce/' . WC_VERSION
            );

            $response = wp_safe_remote_post( $paypal_url, $params );

            if ( ! is_wp_error( $response ) && $response['body'] === 'VERIFIED' ) {
                return true;
            }

            return false;
        }
    }

    add_action( 'valid-paypal-standard-ipn-request', 'handle_valid_ipn_request' );

    function handle_valid_ipn_request( $posted ) {
        if ( ! empty( $posted['custom'] ) ) {
            $order_id = (int) $posted['custom'];
            $order = wc_get_order( $order_id );

            if ( $order && $order->has_status( 'on-hold' ) ) {
                if ( $posted['payment_status'] === 'Completed' ) {
                    $order->payment_complete();
                    $order->add_order_note( 'IPN Payment completed.' );
                } elseif ( $posted['payment_status'] === 'Pending' ) {
                    $order->update_status( 'on-hold', 'IPN Payment pending.' );
                }
            }
        }
    }
}
?>